<?php


session_start();


require_once 'constants.php';
require_once 'utils.php';
require_once 'date.php';
require_once 'security.php';
require_once 'print_tables.php';
require_once 'logger.php';
require_once 'MysqlDB.php';
require_once 'ArchiveScruncher.php';
require_once 'Utils/strutils.php';





function OutputHeader()
{
    echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html>' . "\n";
    echo '<head>' . "\n";
    echo '<link rel="stylesheet" type="text/css" href="styles/style.css" />' . "\n";
    echo '<link rel="stylesheet" type="text/css" href="styles/results.css" />' . "\n";
    echo '<link rel="stylesheet" type="text/css" href="styles/query.css" />' . "\n";
    echo '<link rel="stylesheet" type="text/css" href="styles/atnf.css" />' . "\n";
    echo '' . "\n";
    echo '<meta http-equiv="Content-Type" content="text/html; charset=iso-8759-1" />' . "\n";
    echo '<meta name="DC.Creator" lang="en" content="personalName=Whiting,Matthew" />' . "\n";
    echo '<meta name="ATNF.Maintainer" content="contactName=Webmaster,the" />' . "\n";
    echo '<meta name="DC.Titlepri" lang="en" content="The ATNF Pulsar Archive Searcher" />' . "\n";
    echo '<meta name="DC.Title.alt" lang="en" content="" />' . "\n";
    echo '<meta name="DC.Subject" lang="en" content="pulsar" />' . "\n";
    echo '<meta name="DC.Description" lang="en" content="Search the observation data archives at the ATNF" />' . "\n";
    echo '<meta name="DC.Date.created" scheme="ISO8601" content="2002-12-12" />' . "\n";
    echo '<meta name="ATNF.Site" content="Marsfield" />' . "\n";
    echo '<meta name="ATNF.AccessPermission" content="Public" />' . "\n";
    echo '<meta name="ATNF.MetaVersion" content="1.0" />' . "\n";
    echo '' . "\n";
    echo '<script type="text/javascript" src="scripts/nodomws.js"></script>' . "\n";
    echo '<script type="text/javascript" src="scripts/atnf-css-utils.js"></script>' . "\n";
    echo '<script type="text/javascript" src="scripts/atnf-dom-utils.js"></script>' . "\n";
    echo '<script type="text/javascript" src="scripts/results.js"></script>' . "\n";
    echo '<script type="text/javascript" src="scripts/tooltips.js"></script>' . "\n";
    echo '</head>' . "\n";
    echo '<title>ATNF Pulsar Data Archive Query Results</title>' . "\n";

    include_once("header.html");

    echo '<body class="atoa">' . "\n";
}



/**
 * Add a filename_out parameter to the item given that reflects what the
 * filename we are going to save to should be.
 **/

function AddOutputFilenameToItem( &$item )
{
  // If we don't need any modification, the filename_out is simply the filename

  $item['filename_out'] = $item['filename'];

  // If we are performing any scrunching, add the scrunching parameters to the
  // filename just before the extension.

  if( $item['fscrunch'] != '' ||
      $item['tscrunch'] != '' ||
      $item['pscrunch'] == 'pscrunch' )
  {
    $scr_ext = '_';
    if( $item['fscrunch'] != '' )
      $scr_ext .= 'f' . $item['fscrunch'];
    if( $item['tscrunch'] != '' )
      $scr_ext .= 't' . $item['tscrunch'];
    if( $item['pscrunch'] == 'pscrunch' )
      $scr_ext .= 'p';
    $item['filename_out'] = AddPreExtension( $item['filename'], $scr_ext );
  }

  // If the target file is a directory, ie if the backend was CPSR2, then add
  // .cfb to the output filename

  if( $item['data_type'] == 'CPSR2' )
      $item['filename_out'] .= '.cfb';
}

function OutputFooter()
{
  echo '</form>' . "\n";

  include("footer.html");

  echo '</body>' . "\n";
  echo '</html>' . "\n";
}

function Authenticated()
{
  global $_SESSION;

  if ( !isset($_SESSION['username']) ||
       !isset($_SESSION['auth_code']) ||
       $_SESSION['auth_code'] != OK )
  {

    echo "<center>Your session has expired due to inactivity. Please <a href=\"index.php\">log in</a>.</center>\n";
    return false;
   }

   return true;
}


/**
 * Use the archive parameters to determine the size of the data in the SUBINT table
 **/

function DetermineSubintsSize( $nsub, $nchan, $nbin, $npol )
{
  $bytes_per_int = 2;
  $bytes_per_long = 4;
  $bytes_per_float = 2;
  $bytes_per_double = 4;

  $bytes_per_subint =
      (1 * $bytes_per_long) +
      (8 * $bytes_per_double) +
      (5 * $bytes_per_float) +
      (2 * $nchan * $bytes_per_float) +
      (2 * $nchan * $npol * $bytes_per_float) +
      ($nbin * $nchan * $npol * $bytes_per_int);

  return $nsub * $bytes_per_subint;
}


/**
 * This function takes a guess of what the resulting size of a file will be if we perform
 * the scrunching operations on it. The pre scrunched file may not exist yet because we
 * may not have psradded files together yet. But we still want to know how large the resulting
 * file will be.
 **/

function GuessScrunchedFileSize( $original_size, $nsub, $nchan, $nbin, $npol, $tscrunch, $fscrunch, $pscrunch )
{
  echo "guessing file with $original_size bytes ($nsub,$nchan,$nbin,$npol) <br>";

  $original_data_size = DetermineSubintsSize( $nsub, $nchan, $nbin, $npol );

  $header_size = $original_size - $original_data_size;

  if( $pscrunch == 'pscrunch' )
    $npol = 1;
  if( $tscrunch != '' )
    $nsub = ceil( $nsub / $tscrunch );
  if( $fscrunch != '' )
    $nchan = ceil( $nchan / $fscrunch );

  $new_data_size = DetermineSubintsSize( $nsub, $nchan, $nbin, $npol );

  $new_file_size = $new_data_size + $header_size;

  echo "will scrunch to $new_file_size bytes ($nsub,$nchan,$nbin,$npol) <br>";

  return $new_file_size;
}


/**
 * If the file doesn't have any scrunching return the file_size_bytes field.
 * But if it does require scrunching, create a scruncher object and get it to
 * make a guess as to how large the resulting file will be.
 **/

function DetermineDownloadSize( $file_item, $db )
{
  $src_file = new File( $file_item['data_loc'], $file_item['filename'] );

  $size = $file_item['file_size_bytes'];

  // If the item is CPSR2 data, we need to figure out what the size of the combined
  // files will be.

  if( $file_item['pscrunch'] == 'pscrunch' ||
      $file_item['tscrunch'] == '' ||
      $file_item['fscrunch'] == '' )
  {
    $size = GuessScrunchedFileSize( $size,
				    $file_item['nsub'],
				    $file_item['nchan'],
				    $file_item['nbin'],
				    $file_item['npol'],
				    $file_item['tscrunch'],
				    $file_item['fscrunch'],
				    $file_item['pscrunch'] );
  }

  // get a list of any calibration files associated with this item

  if( $file_item['cals_within_mins'] != 'off' )
  {
    $cals_nearby = query_cals( $src_file->GetFullPath(),  $file_item['cals_within_mins'], $db );
    foreach( $cals_nearby as $next_cal )
      $size += $next_cal['file_size_bytes'];
  }

  return $size;
}




// Determine the range of items we are going to display by doing the following;
//
// take the range from get vars if available
// if we still don't have a range
//   take it from session data
// if we still don't have a range
//   set the range to go from 1 to -1
// if this is a cart
//   use the cart_items as the target array
// otherwise
//   use the results set as the target array
// if there is nothing in the target array
//   set the range to 1 - 1
// else
//   if the range starts at more than the number of items in the target array
//     set the range to start at 1
//   if the range ends at -1 or more than the number of items in the target array
//     set the range to end at the number of target array
// if the range is more than the max number per page
//   reduce the range to the max number per page
// if the range is less than max number per page
//   if start plus max per page is less than the number in target array
//     set the range to end at start plus max per page
//   otherwise
//     set the range to end at the number in the target array

function DetermineRange()
{
  // TODO need to reduce these globals, especially target_count
  global $startRange;
  global $endRange;
  global $target_count;
  global $_SESSION;
  global $HTTP_GET_VARS;
  global $cart_items;
  global $all_results;
  global $state;

  $startRange  = $HTTP_GET_VARS['startRange'];
  $endRange = $HTTP_GET_VARS['endRange'];	

  if (!isset($startRange) || !isset($endRange)) {
    $startRange = $_SESSION['startRange'];
    $endRange = $_SESSION['endRange'];
  }

  if (!isset($startRange) || !isset($endRange)) {
    $startRange = 1;
    $endRange = -1;
  }

  $target_count = 0;
  if( $state == 'cart' ) {
    $target_count = count( $cart_items );
  }
  else {
    $target_count = count( $all_results );
  }

  if( $target_count == 0 ){
    $startRange = 1;
    $endRange = -1;
  }
  else {
    if( $startRange > $target_count )
      $startRange = 1;
    if( $endRange == -1 || $endRange > $target_count )
      $endRange = $target_count;
  }

  if( $endRange - $startRange > MAX_DISPLAY_ROWS ) {
    $endRange = $startRange + MAX_DISPLAY_ROWS - 1;
  }

  if( $endRange - $startRange < MAX_DISPLAY_ROWS )
  {
    if( $startRange + MAX_DISPLAY_ROWS < $target_count )
      $endRange = $startRange + MAX_DISPLAY_ROWS - 1;
    else
      $endRange = $target_count;
  }
}



/**
 * main
 **/

function main()
{
  global $_SESSION;
  global $HTTP_GET_VARS;
  global $params, $params_in, $params_URL,
	  $pulsar_name, $orderBy, $data_type, $orderDir, $bandname,
	  $rahour, $ramin, $rasec, $decdeg, $decmin, $decsec,
	  $radius, $from_day,	$from_month, $from_year,
	  $to_day, $to_month, $to_year, $cals, $cals_within_mins, $state,
	  $cart_items;
  global $startRange, $endRange, $target_count, $all_results;
  
  OutputHeader();

  try
  {
    $db = new MysqlDB();
    $db->connect( DB_SERVER, DB_USERNAME, DB_PASS, DB_NAME );

    if( Authenticated() )
    {
      // TODO see if this is necessary
      // Clean out any old variables
      init();
	
///////////////////////////
// Collect the form values

// List of request parameters to display
$params_in        = get_param('params', $HTTP_GET_VARS);
$_SESSION['fields'] = $params_in;


$params_URL       = array2urlstring("params", $params_in);	
	
// The current state. E.g cart or results
// Need to maintain this because other actions could be performed without changing state 
$state            = get_param('state', $HTTP_GET_VARS);
	
// The selected action to perform. E.g. add observations to cart, or view profile
$actions          = get_param('actions', $HTTP_GET_VARS);
	
// List of checked observations. Note: Not necessarily observations in the cart.
// Cart observations are stored in $cart_items variable
$selected_obs     = get_param('selected_obs', $HTTP_GET_VARS);
	
// get the cals, but don't use get_param, because if it isn't set in HTTP_GET_VARS then
// we don't fetch it from session, but take it as unset. ie if a checkbox isn't set
// it won't appear in HTTP_POST_VARS. Also, we still save it to session because the
// index.php page needs to know what we last used if we click on "new query"
$cals = '';
if( isset( $HTTP_GET_VARS['cals'] ) )
    $cals = $HTTP_GET_VARS['cals'];
$_SESSION['cals'] = $cals;


	
// an array mapping observation number (corresponding to the index in all_results) 
// to an array of matching cals
$obs_cals         = array();
		
// Display cals within this many minutes of the observation
$cals_within_mins = get_param('cals_within_mins', $HTTP_GET_VARS);
	
// cart_items is an array of numbers that index to the list of 
// results in all_results
$cart_items       = $_SESSION['cart_items'];
	
	
// this holds all the query results of observations as an array of
// associative arrays. E.g.
// array ( 0 => ( pulsar_name => 'J0437-4715',
//                DM          => 10,
//                period      => 1
//              ),
//
//         1 => (pulsar_name => 'J0437-4715',
//                DM          => 27,
//                period      => 1.112
//              )
//       )
//
if (isset($_SESSION['results'])) {
  $all_results = $_SESSION['results'];
 }
	
	
// Other form parameters. Pretty self explanatory?
$pulsar_name      = get_param('pulsar_name', $HTTP_GET_VARS);
$project_id       = get_param('projID', $HTTP_GET_VARS);
$obs_mode         = get_param('obs_mode', $HTTP_GET_VARS);
$data_type        = get_param('data_type', $HTTP_GET_VARS);
$orderBy          = get_param('orderBy', $HTTP_GET_VARS);
$orderDir         = get_param('orderDir', $HTTP_GET_VARS);
$survey           = get_param('survey', $HTTP_GET_VARS);
$rahour           = get_param('rahour', $HTTP_GET_VARS);
$ramin            = get_param('ramin', $HTTP_GET_VARS);
$rasec            = get_param('rasec', $HTTP_GET_VARS);
$decdeg           = get_param('decdeg', $HTTP_GET_VARS);
$decmin           = get_param('decmin', $HTTP_GET_VARS);
$decsec           = get_param('decsec', $HTTP_GET_VARS);
$radius           = get_param('radius', $HTTP_GET_VARS);
$bandname         = get_param('bandname', $HTTP_GET_VARS);
$from_day         = get_param('from_day', $HTTP_GET_VARS);
$from_month       = get_param('from_month', $HTTP_GET_VARS);
$from_year        = get_param('from_year', $HTTP_GET_VARS);
$to_day           = get_param('to_day', $HTTP_GET_VARS);	
$to_month         = get_param('to_month', $HTTP_GET_VARS);
$to_year          = get_param('to_year', $HTTP_GET_VARS);
$pscrunch         = get_param('pscrunch', $HTTP_GET_VARS);
$tscrunch         = get_param('tscrunch', $HTTP_GET_VARS);
$fscrunch         = get_param('fscrunch', $HTTP_GET_VARS);

$pscrunch_checked = '';
if( $pscrunch == 'pscrunch' ) $pscrunch_checked = 'checked';

if( $bandname == '' )
  $bandname = Array( 'anyBand' );



	
$tar_file = "";

$psrNameOK = ($pulsar_name != "");

if(!$psrNameOK){ // we need the RA and Dec values
  $raj = ($rahour + $ramin/60. + $rasec/3600.) * 15.;
  $decj = abs($decdeg) + $decmin/60. + $decsec/3600.;
  if($decdeg[0]=='-') {
    $decj *= -1.;
  }
 }

$params = array();
$params[0] = "distance";
for ($i=0; $i < count($params_in); $i++) {
  array_push($params,$params_in[$i]);
 }


///////
// Do some basic error checking: SR>0, |dec|<90, ...
// echo "state=$state<br>\n";
// echo "actions=$actions<br>\n";
// echo "size of _SESSION['results']= ".count($_SESSION['results'])."<br>\n";
if(!isset($actions)){

  $isError = False;
  $errorString = "";
  
  if($radius==""){		//	if(!isset($radius)){
    $isError=True;
    $errorMsg .= "<center><b>Error:</b> You have not provided a search radius. <br>\n";
  }
  if(($radius!="") && !is_numeric($radius)){
    $isError=True;
    $errorMsg .= "<center><b>Error:</b> Your search radius value (" . $radius . ") is not a number. <br>\n";
  }
  // If SR is negative, this is bad.
  if( is_numeric($radius) && ($radius < 0.) ){
    $isError = True;
    $errorMsg .= "<center><b>Error:</b> Your search radius value (" . $radius . ") is negative. <br>\n";
  }
  // If DEC is out of range, this is bad.
  if( is_numeric($decj) && (abs($decj) > 90.) ){
    $isError = True;
    $errorMsg .= "<center><b>Error:</b> Your DEC value (" . $decj . ") is out of range. <br>\n";
  }
  
 }
 





// Calculate the total size of observations cart
$total_cart_size_bytes = 0;

if( isset( $cart_items ) && is_array( $cart_items ) )
  foreach ($cart_items as $item) 
    $total_cart_size_bytes += DetermineDownloadSize($item, $db);
else
  $cart_items = array();


///////////////////////
// Perform actions
//
// Precondition: The query has already been made, so the list of cals foreach observation
// has already been determined ($_SESSION['obs_cals']).

if (isset($actions) && $actions != "") 
  {
  
    switch ($actions) 
      {

      case DOWNLOAD_ALL_OBS_CALS:
	if (isset($cart_items) && $total_cart_size_bytes < MAX_DOWNLOAD_BYTES) {
	  foreach( $cart_items as $next_item )
	       logger_download( $_SESSION['username'],
				$next_item['data_loc'],
				$next_item['filename'], 
				$next_item['file_size_bytes'] ); 
	  $tar_file = tarFiles($cart_items, $all_results, $db, true, true );
	  echo " <meta http-equiv=\"REFRESH\" content=\"0; URL=".WEB_DOWNLOADS_DIR."/$tar_file\">\n";
	}

	break;
				
      case DOWNLOAD_ALL_OBS:

	if (isset($cart_items) ) {
	  foreach( $cart_items as $next_item )
	    logger_download( $_SESSION['username'],
			     $next_item['data_loc'],
			     $next_item['filename'], 
			     $next_item['file_size_bytes'] ); 
	  //echo "Download all<br>\n";
	  $tar_file = tarFiles($cart_items, $all_results, $db, true, false );
	  echo " <meta http-equiv=\"REFRESH\" content=\"0; URL=".WEB_DOWNLOADS_DIR."/$tar_file\">\n";
	}

	break;
			
      case DOWNLOAD_ALL_CALS:

	if (isset($cart_items) ) {
	  foreach( $cart_items as $next_item )
	    logger_download( $_SESSION['username'],
			     $next_item['data_loc'],
			     $next_item['filename'], 
			     $next_item['file_size_bytes'] ); 
	  //echo "Download all<br>\n";
	  $tar_file = tarFiles($cart_items, $all_results, $db, false, true );
	  if( $tar_file != '' )
	    echo " <meta http-equiv=\"REFRESH\" content=\"0; URL=".WEB_DOWNLOADS_DIR."/$tar_file\">\n";
	  else
	    echo "<div class=exception_box> No cal files associated with the observations in your cart. </div>";
	}

	break;
			
      case VIEW_PROFILES:
					
	if (isset($selected_obs) && isset($all_results)) {
					
	  // redirect to the view_profile page and pass through the selected_obs and obs_cals arrays
	  $_SESSION['selected_obs'] = $selected_obs;
	  $_SESSION['display_cals'] = false;
					
	  include_once("view_profiles.php");

	  exit;
	}

	break;
			
      case VIEW_PROFILES_CALS:
				
	if (isset($selected_obs) && isset($all_results)) {
					
	  // redirect to the view_profile page and pass through the selected_obs and obs_cals arrays
	  $_SESSION['selected_obs'] = $selected_obs;
	  $_SESSION['display_cals'] = true;
					
	  include_once("view_profiles.php");
	  exit;
	}

	break;
			
      case REMOVE_FROM_CART:
	if (isset($selected_obs)) {
	    $items_to_remove = Array();
	    foreach( $selected_obs as $next_index )
	      $items_to_remove[] = $cart_items[$next_index];
	    $cart_items = array_diff2($cart_items, $items_to_remove);
	    // update the file size estimate
            $total_cart_size_bytes = 0;
	    foreach ($cart_items as $item) 
	      $total_cart_size_bytes += DetermineDownloadSize($item, $db);
	}
	break;

      case ADD_TO_CART:
	if (isset($selected_obs)) 
	{
	  // Get a list of item entries from the result set that match our list of selected objects
	  $items_to_add = Array();
	  foreach( $selected_obs as $next_index )
	    $items_to_add[] = $all_results[$next_index];
	  
	  for( $ind = 0; $ind < count( $items_to_add ); $ind ++ ) 
	  {
	      // Add the scrunching parameters to the item
	      $items_to_add[$ind]['pscrunch'] = $pscrunch;
	      $items_to_add[$ind]['tscrunch'] = $tscrunch;
	      $items_to_add[$ind]['fscrunch'] = $fscrunch;

	      AddOutputFilenameToItem( $items_to_add[$ind] );

	      // Add the cals_within_mins form parameter to the item
	      if( $cals == 'cals' )
	        $items_to_add[$ind]['cals_within_mins'] = $cals_within_mins;
	      else
		$items_to_add[$ind]['cals_within_mins'] = 'off';
	  }
	  
	  // Save the items in the cart array
	  foreach ($items_to_add as $ob) {
	    if (!in_array($ob, $cart_items)) {
	      array_push($cart_items, $ob);
	    }
	  }
	  // update our file size estimate
	  $total_cart_size_bytes = 0;
	  foreach ($cart_items as $item) 
	    $total_cart_size_bytes += DetermineDownloadSize($item, $db);
	}
	break;
      }
  }



DetermineRange();

////////////////////////////////////////////////////////////////////

	
$_SESSION['cart_items'] = $cart_items;
$_SESSION['startRange'] = $startRange;
$_SESSION['endRange'] = $endRange;
	

if (!isset($params) && !isset($_SESSION['results'])) 
  {
    echo "No parameters were selected. Please select at least one.<br>\n";
    include("footer.html");
    echo "</body>\n</html>\n";
    exit();
  }
	
if (!isset($_SESSION['results']) || $HTTP_GET_VARS['submit_button'] == 'submit') {
		
  /////////////////////////////////////////////////////
  // Form the parameter string in the SQL SELECT clause
  $parameters = $params_in[0];

  for ($i=1; $i < count($params_in); $i++) {
    $parameters .= ", " . $params_in[$i];
  }

  $parameters .= ",data_loc,file_size_bytes,concat(data_loc,'/',filename) as pathname";

  // remove any leading and trailing commas
  $parameters = trim($parameters, ",");


  ///////////////////////////////////////
  // Convert gregorian day to julian day
  //

  $from_mjd = cal_to_jd (CAL_GREGORIAN, $from_month,  $from_day, $from_year ) - 2400000.5;
  $to_mjd = cal_to_jd ( CAL_GREGORIAN, $to_month, $to_day, $to_year ) - 2400000.5;

  /////////////////////////////////////////
  // Check that the user hasn't queried for data younger than 18 months
  //
		
  if (!atnf_staff($_SESSION['username'])) {
  //  if($authenticationType = "anon"){

    $today_mjd = cal_to_jd ( CAL_GREGORIAN, date("n"), date("j"), date("Y") ) - 2400000.5;
    $max_mjd = $today_mjd - PROPRIETRY_PERIOD_DAY;
    $max_date = cal_from_jd($max_mjd+2400000.5, CAL_GREGORIAN);

    if ($max_mjd < $from_mjd) {  // no overlap in date range -- no data returned
      echo "<p>Your requested date range does not overlap with the non-proprietary period, which ends <b>".
	$max_date['day']." ".$max_date['abbrevmonth'].", ".$max_date['year']."</b>.</p>\n";
      $isError = True;
    }
    else if($max_mjd < $to_mjd) {
      echo "<p>Your requested maximum date is in the proprietary period. Changing it to <b>".
	$max_date['day']." ".$max_date['abbrevmonth'].", ".$max_date['year']."</b>.</p>\n";
      $to_mjd = $max_mjd;
    }

    
  }
		
  if(!$isError && !isset($actions))  {

    ///////////////////////////////
    // Set up the query string
    //

    // using "1 = 1" is a hack so we don't have to bother making sure
    // the whereString doesn't start with "AND"
    $whereString = " 1 = 1 "; 
    $selectString = "SELECT ";

    ///////////////////////////
    // Constraint on pulsar name
    //
    //  In this case, query on pulsar name and on pulsar position 
    //       -- obtain the latter from PSRCAT
    //  if ($pulsar_name) {
    if ($psrNameOK) {

      // Check to see if the pulsar given is in the catalogue
      $command = PSRCAT." -nohead -nonumber -o short -c \"\" $pulsar_name";
      exec($command,$output);

      if($output[0]=="") { // Pulsar is present

	$commandPSR = PSRCAT." -nohead -nonumber -o short -c \"RAJD DECJD\" $pulsar_name";
	exec($commandPSR, $outputPSR);
	list($psr_rajd, $psr_decjd) = split("[ ]+", trim($outputPSR[0]));
            
	// Remove the leading J and B
	$output_psrname = ltrim($pulsar_name, "JBbj");

	$angsepStringPSR = "degrees( acos( cos(radians(rajd-$psr_rajd))*cos(radians(decjd))*cos(radians($psr_decjd))+sin(radians(decjd))*sin(radians($psr_decjd))) )";
      
	$whereString .= " AND ( (($angsepStringPSR - 0.25*(bmaj+bmin)) < ($radius/60.)) OR src_name LIKE '%$output_psrname' )";
      
	$selectString .= "($angsepStringPSR*60.) as distance, ";
      
      }
      else{
	$whereString .= "AND (1 = 0)"; // return nothing;
	$selectString .= "0 as distance, ";
	$psrNameOK = False;

	$isError = True;
	$errorMsg .= "<center><b>Error:</b> The pulsar name given, <b>$pulsar_name</b>, is not in the catalogue</center><br>\n";
      }


    }
    else{// if ($radius) {
      ////////////////////////////
      // Constraint on boundary
      //  The pulsar name search takes precedence if both fields are filled in.

      $angsepString = "(degrees( acos( cos(radians(rajd-$raj))*cos(radians(decjd))*cos(radians($decj))+sin(radians(decjd))*sin(radians($decj))) ) )";
		  
      $whereString .= " AND (($angsepString - 0.25*(bmaj+bmin)) < ($radius/60.)) ";
		  
      $selectString .= "($angsepString*60.) as distance, ";

    }

    $selectString .= $parameters;

    ////////////////////////////
    // Constraint on project ID

    if ($project_id) {
      $whereString .= " AND project_id = '$project_id'";
    }

    ////////////////////////////
    // Constraint on obs mode

    if ($obs_mode == "search") {
      $whereString .= " AND filename like '%.sf'";
    }
    else if($obs_mode == "timing"){
      $whereString .= " AND filename not like '%.sf'";
    }

    ////////////////////////////
    // Constraint on data type

    if ($data_type) {
      $whereString .= " AND data_type = '$data_type'";
    }

    ////////////////////////////
    // Constraint on survey

    if ($survey) {
      $whereString .= " AND survey = '$survey'";
    }

    //////////////////////
    // Constraint on MJD

    $whereString .= " AND (MJD < $to_mjd AND MJD > $from_mjd)";

    //////////////////////
    // Constraint on freq

    $bandtext = Array("70cm" => "obsfreq<500.",
		      "50cm"  => "obsfreq>600. and obsfreq<700.",
		      "20cm"  => "obsfreq>1199.99 and obsfreq<1800.01",
		      "13cm"  => "obsfreq>1999.99 and obsfreq<2500.01",
		      "10cm"  => "obsfreq>2599.99 and obsfreq<3600.01",
		      "6cm"   => "obsfreq>4499.99 and obsfreq<5100.01",
		      "5cm"   => "obsfreq>5899.99 and obsfreq<6800.01",
		      "3cm"   => "obsfreq>8099.99 and obsfreq<8600.01",
		      "1.3cm" => "obsfreq>20999.99 and obsfreq<24000.01");
    
    $bandString = " AND ( (" . $bandtext[$bandname[0]];
    for ($i=1; $i < count($bandname); $i++) {
      $bandString .= ") OR (" . $bandtext[$bandname[$i]];
    }
    $bandString .= ") )";

    if(count($bandname)==0){
      $bandString = "";
    }

    for ($i=0; $i < count($bandname); $i++) {
      if($bandname[$i] == "anyBand") $bandString = "";
    }
    $whereString .= $bandString;
    
    ////////////////////////
    //  Ordering the results
  
    $orderString = " ORDER BY ";
    $orderString .= $orderBy . " " . $orderDir;

    // add other alternatives, in order of preference, if they haven't been selected first.
    if($orderBy!="distance") $orderString .= ",distance asc";
    if($orderBy!="filename") $orderString .= ",filename desc";
    if($orderBy!="date")     $orderString .= ",date asc";
    if($orderBy!="obsfreq")  $orderString .= ",obsfreq asc";


    ///////////////////////////////
    // Performing SQL query
    //



    $query = "( $selectString FROM ".S70_TABLE." WHERE $whereString ) UNION ( $selectString FROM ".OBSERVATIONS_TABLE." WHERE $whereString )";
    
    $query .= $orderString;

    $query .= " limit ".MAX_RESULTS; // limit the maximum number of results to be returned.

    // get the number of results we would get without limiting it.
    // and give a warning if we're not returning all of them.
    $num_query = "select a.num+b.num as num from (select count(*) as num from ".S70_TABLE." where $whereString) as a, (select count(*) as num from ".OBSERVATIONS_TABLE." where $whereString) as b";

    $num_result = $db->DoQuery( $num_query );
    $row = $db->FetchRowAsArray();
    if( $row['num'] > MAX_RESULTS && $state == 'results' )
      $_SESSION['num_msg'] =
	   "<center>\n" .
	   "<b>Note:</b> Your query returns more results (".$row['num'].") than the internal limit.<br>\n" .
	   "Only showing the first ".MAX_RESULTS." rows.\n" .
	   "</center><br>\n";

    $result = $db->DoQuery($query);
    $all_results = $db->GetLastResult();

    logger_query( $_SESSION['username'], $query, $db->FetchNumRows() );


	       
    $_SESSION['results'] = $all_results;
    
    DetermineRange();
	       

  }
 }
	
// Make a copy of it to pass to another function
$params_cpy = $params;

// Get the number of rows and report
$num_rows = count($all_results);


echo "<form action=\"" . $PHP_SELF . "\" method=\"GET\" id=\"frmResults\">\n";

foreach($params_in as $par){
  echo "<input type=\"hidden\" name=\"params[]\" value=\"".$par."\">\n";
}
echo "<input type=\"hidden\" name=\"startRange\" value=\"".$startRange."\">\n";
echo "<input type=\"hidden\" name=\"endRange\" value=\"".$endRange."\">\n";
echo "<input type=\"hidden\" name=\"pulsar_name\" value=\"".$pulsar_name."\">\n";
echo "<input type=\"hidden\" name=\"obs_mode\" value=\"".$obs_mode."\">\n";
echo "<input type=\"hidden\" name=\"data_type\" value=\"".$data_type."\">\n";
echo "<input type=\"hidden\" name=\"orderBy\" value=\"".$orderBy."\">\n";
echo "<input type=\"hidden\" name=\"orderDir\" value=\"".$orderDir."\">\n";
echo "<input type=\"hidden\" name=\"survey\" value=\"".$survey."\">\n";
echo "<input type=\"hidden\" name=\"rahour\" value=\"".$rahour."\">\n";
echo "<input type=\"hidden\" name=\"ramin\" value=\"".$ramin."\">\n";
echo "<input type=\"hidden\" name=\"rasec\" value=\"".$rasec."\">\n";
echo "<input type=\"hidden\" name=\"decdeg\" value=\"".$decdeg."\">\n";
echo "<input type=\"hidden\" name=\"decmin\" value=\"".$decmin."\">\n";
echo "<input type=\"hidden\" name=\"decsec\" value=\"".$decsec."\">\n";
echo "<input type=\"hidden\" name=\"radius\" value=\"".$radius."\">\n";
 if(count($bandname)==0){
   echo "<input type=\"hidden\" name=\"bandname[]\" value=\"\">\n";
 }
 else{
   foreach($bandname as $band){
     echo "<input type=\"hidden\" name=\"bandname[]\" value=\"".$band."\">\n";
   }
 }
echo "<input type=\"hidden\" name=\"from_day\" value=\"".$from_day."\">\n";
echo "<input type=\"hidden\" name=\"from_month\" value=\"".$from_month."\">\n";
echo "<input type=\"hidden\" name=\"from_year\" value=\"".$from_year."\">\n";
echo "<input type=\"hidden\" name=\"to_day\" value=\"".$to_day."\">\n";
echo "<input type=\"hidden\" name=\"to_month\" value=\"".$to_month."\">\n";
echo "<input type=\"hidden\" name=\"to_year\" value=\"".$to_year."\">\n";
echo "<input type=\"hidden\" name=\"cals\" value=\"".$cals."\">\n";
echo "<input type=\"hidden\" name=\"cals_within_mins\" value=\"".$cals_within_mins."\">\n\n";

echo "<table class=\"top_menu\" border=\"0\" id=\"tabQueryResults\">\n";
if ($state == 'cart') 
  { 
    //    echo " <tr class=\"top_menu\"><td class=\"top_menu_heading\" colspan=\"4\"><h2>Observations Cart</h2></td></tr>\n";
    echo " <tr class=\"top_menu\"><td colspan=\"4\"><h2>Observations Cart</h2></td></tr>\n";
  } 
 else if ($state == 'results') 
   { 
     //     echo " <tr class=\"top_menu\"><td class=\"top_menu_heading\" colspan=\"4\"><h2>Query Results</h2></td></tr>\n";
     echo " <tr class=\"top_menu\"><td  colspan=\"4\"><h2>Query Results</h2>\n";
     if($_SESSION['num_msg'] != ""){
       echo $_SESSION['num_msg'];
     }
     echo "</td></tr>\n";
   } 

echo " <tr class=\"top_menu\">\n";
echo "  <td class=\"top_menu\"  align=\"left\" width=\"30%\">\n";
echo "    <a class=\"top_menu\" href=\"index.php?command=logout\">Logout</a> | \n";
echo "    <a class=\"top_menu\" href=\"index.php?actions=new_query\">New Query</a>\n";
echo "  </td>\n";

echo "  <td class=\"top_menu\" width=\"50%\" align=\"center\">\n";
echo "    Actions: <select name=\"actions\">\n";

if ($state == "cart") 
  { 
    echo "      <option>--------- Select an action --------</option>\n";
    if ( ($total_cart_size_bytes/1000000) < MAX_DOWNLOAD_MB ) {
      echo "      <option>".DOWNLOAD_ALL_OBS_CALS."</option>\n";
      echo "      <option>".DOWNLOAD_ALL_OBS."</option>\n";
      echo "      <option>".DOWNLOAD_ALL_CALS."</option>\n";
    }
    else {
      echo "      <option disabled>Download limit ".MAX_DOWNLOAD_MB."MB exceeded</option>\n";
    }
    echo "      <option>".REMOVE_FROM_CART."</option>\n";
    echo "    </select>\n";
    echo "    <input type=\"submit\" value=\"Perform action\">\n";
    echo "  </td>\n  <td class=\"top_menu\" align=\"right\">\n";
  
    echo "    Cart size: <b><a href=\"";
    printURL("cart",1,count($cart_items));
    echo "\">".count($cart_items);
    $sizeMB = sprintf("%.1f MB", $total_cart_size_bytes/1000000);
    echo " Item/s</a> (".$sizeMB.")\n  </td>\n";
  
    echo "  <td class=\"top_menu\" align=\"right\" width=\"1%\">\n";
    echo "    <a href=\"";
    printURL("results",1,MAX_DISPLAY_ROWS);
    echo "\">";
    echo "<img alt=\"Revert to query results\" border=\"0\" title=\"Revert to query results\" src=\"/images/table_30px_white.png\"></img>\n";
    echo "    </a>\n  </td>\n";
  }
 else 
   { 
     echo "      <option>--------- Select an action --------</option>\n";
     echo "      <option>".ADD_TO_CART."</option>\n";
     echo "      <option>----- Profile Viewing Options -----</option>\n";
     echo "      <option>".VIEW_PROFILES."</option>\n";
     if (isset($cals) && ($cals != "")) {
       echo "      <option>".VIEW_PROFILES_CALS."</option>\n";
     }
     echo "    </select>\n";
     echo '    <a href="#" onClick="toggleScrunchingParameters();return false;">Reduce Data</a>';
     echo '    <div id="divScrunching" style="display:none" >';
     echo "    Sum every <input type=text name=fscrunch size=4/ value=\"$fscrunch\"> frequency channels together <br>";
     echo "    Sum every <input type=text name=tscrunch size=4/ value=\"$tscrunch\"> sub integration together <br>";
     echo "    <input type=checkbox name=pscrunch value=pscrunch $pscrunch_checked /> Reduce polarization to total intensity <br>";
     echo '    </div>';
     echo '    <br>';
     echo "    <input type=\"submit\" value=\"Perform action\">\n";
     echo "  </td>\n";
     echo "  <td class=\"top_menu\" align=\"right\" width=\"20%\">\n";
     $sizeMB = sprintf("%.1f MB", $total_cart_size_bytes/1000000);
     echo "    Cart size: <b><a href=\"";
     printURL("cart",1,count($cart_items));
     echo "\">".count($cart_items);
     echo " Item/s</a> (".$sizeMB.")\n";
     echo "  </td>\n";
     echo "  <td class=\"top_menu\" align=\"right\" width=\"1%\">\n";
     echo "    <a href=\"";
     printURL("cart",1,count($cart_items));
     echo "\">";
     echo "<img alt=\"View observation cart\" border=\"0\" title=\"View observation cart\" src=\"/images/cart_30px_white.png\"></img>\n";
     echo "    </a>\n  </td>\n";
 }
echo " </tr>\n</table>\n";

if ($endRange ==  -1) 
  {
    if($state == "cart"){
      echo "<center>Your cart is empty.</center><br>\n";
      echo "<center><a href=\"";
      printURL("results",1,MAX_DISPLAY_ROWS);
      echo "\">Go back to search results</a></center><br>\n";
    }
    else if(!isset($actions)) {
      if($isError){
	echo $errorMsg;
      }
      else{
	echo "<center>Sorry, no results matched your search</center><br>\n";
      }
      echo "<center><a href=\"index.php\">Search again</a></center><br>\n";
    }
    echo "</form>\n";
    include("footer.html");
    echo "</body>\n</html>\n";
    exit();
  }

  
echo "<center>\n";
printNavigate($startRange, $target_count);
echo "</center>\n";				
	

echo "Showing <b>$startRange - $endRange</b> of <b>$target_count</b> rows\n";

	
showTable($startRange-1, $endRange-1, $heading, $all_results, $params_cpy, $cals, $db);
	
echo "<center>\n";
printNavigate($startRange, $target_count);
echo "</center>\n";
    }

} catch( MysqlDBException $e ) {
  logger_exception( $_SESSION['username'], $e->getCode(), $e->getMessage(), '' );
  echo '<p><div class="exception_box">' .
      '<center>' .
      '<b>A database error was encountered</b> <br>' .
      'the details of the error have been logged, please contact the system administrator for futher assistance <br>' .
      '</center>' .
      '</div><p>';
}
  OutputFooter();
} // main

main();

?>
